#-------------------------------------------------------------------------------
#. Universidade Estadual de Santa Cruz - UESC
#. Ttulo: Gerao de gifs para as principais distribuies contnuas
#. Curso : Introduo  estatstica
# Grupo : Ian Andrade Moreira
#         Johnatas Barbosa dos Santos
# Data  : 2024-06-22 22:07:31 (reviso - J.C.Faria)
#-------------------------------------------------------------------------------

# 'magick': instalao do pacote necessrio para a gerao de gifs
install.packages('magick')

# 'magick': carregando o pacote
library(magick)


# Observado o diretrio de trabalho
# ( para onde sero exportados os arquivos .gif)
getwd()

#. Distribuio normal
m <- 10  # mdia

nc <- seq(1,
          15,
          by=1)

cores <- rainbow(length(nc))              

img <- image_graph(res=100)
par(mar=c(5, 5, 4, 2))

out <- lapply(nc,
              function(i){
  plot(NA,
       xlab='x',
       ylab='f(x)',
       xlim=c(-20, 40),
       ylim=c(0, 0.4),
       type='n',
       cex.lab=1.5)

  curve(dnorm(x,
              m=m,
              sd=i),
        col=cores[i],
        lwd=3,
        add=TRUE,
        n=1e3)

  temp <- legend("topright",
                 legend=c(expression(paste(mu,
                                           '=')),
                          expression(paste(sigma,
                                           '='))),
                 text.width=5,
                 title="Parmetros",
                 title.cex=0.5)

  text(x=temp$rect$left + temp$rect$w,
       y=temp$text$y,
       labels= c(m, i),
       pos=2)

  title('Distribuio Normal')
})

# Aguarde finalizar...
dev.off()

animation <- image_animate(img,
                           fps=2)

image_write(animation,
            'fdp_Normal.gif')

                    
#. Distribuio t de Student
nc <- seq(1,
          30,
          by=1)

cores <- rainbow(length(nc))              

img <- image_graph(res=100)
par(mar=c(5, 5, 4, 2))

out <- lapply(nc,
              function(i){
  plot(NA,
       xlab='t',
       ylab='f(t)',
       xlim=c(-10, 10),
       ylim=c(0, 0.4),
       type='n',
       cex.lab=1.5)

  curve(dt(x,
           df=i),
        col=cores[i],
        lwd=3,
        add=TRUE,
        n=1e3)        

  temp <- legend("topright",
                 legend=expression(paste(phi,
                                         '=')),
                 text.width=1.5,
                 title="Parmetros",
                 title.cex=0.5)

  text(x=temp$rect$left + temp$rect$w,
       y=temp$text$y,
       labels=i,
       pos=2)

  title('Distribuio t de Student')
})

# Aguarde finalizar...
dev.off()

animation <- image_animate(img,
                           fps=2)

image_write(animation,
            'fdp_Student.gif')
                    
#. Distribuio Qui-Quadrado
nc <- seq(1,
          40, 
          by=1)

cores <- rainbow(length(nc))              

img <- image_graph(res=100)
par(mar=c(6, 5, 4, 2))

out <- lapply(nc,
              function(i){
  plot(NA,
       xlab=expression(chi^2),
       ylab=expression(f(chi^2)),
       xlim=c(0, 80),
       ylim=c(0, 0.5),
       type='n',
       cex.lab=1.2)

  curve(dchisq(x,
           df=i),
        col=cores[i],
        lwd=3,
        add=TRUE,
        n=1e3)        

  temp <- legend("topright",
                 legend=expression(paste(varphi,
                                         '=')),
                 text.width=5,
                 title="Parmetros",
                 title.cex=0.5)

  text(x=temp$rect$left + temp$rect$w,
       y=temp$text$y,
       labels=i,
       pos=2)

  title('Distribuio Qui-Quadrado')
})

# Aguarde finalizar...
dev.off()

animation <- image_animate(img,
                           fps=2)

image_write(animation,
            'fdp_Qui-Quadrado.gif')

                    
#. Distribuio F
df2 <- 15  # Graus de liberdade do denominador
nc <- seq(1,
          50,
          by=1)

cores <- rainbow(length(nc))              

img <- image_graph(res=100)
par(mar=c(5, 5, 4, 2))

out <- lapply(nc,
              function(i){
  plot(NA,
       xlab='F',
       ylab='f(F)',
       xlim=c(0, 10),
       ylim=c(0, 1.2),
       type='n',
       cex.lab=1.5)

  curve(df(x,
           df1=i,
           df2=df2),
        col=cores[i],
        lwd=5,
        add=TRUE,
        n=1e3)        

  temp <- legend("topright",
                 legend=c(expression(paste(varphi[1],
                                           '=')),
                          expression(paste(varphi[2],
                                           '='))),
                 text.width=1,
                 title="Parmetros",
                 title.cex=0.5)

  text(x=temp$rect$left + temp$rect$w,
       y=temp$text$y,
       labels= c(i, df2),
       pos=2)

  title('Distribuio F')
})

# Aguarde finalizar...
dev.off()

animation <- image_animate(img,
                           fps=2)

image_write(animation,
            'fdp_F.gif')
