#-----------------------------------------------------------------------
#. Universidade Estadual de Santa Cruz - UESC
#. Ttulo: Variveis aleatrias contnuas - VAC
#. Curso : Introduo  estatstica
# Grupo: Ian Andrade Moreira
#        Johnatas Barbosa dos Santos
# Baseado no script desenvolvido por: Jos Cludio Faria/UESC/DCEX
# Revisado: 2024-07-02 21:03:09 por: Jos Cludio Faria/UESC/DCEX
#-----------------------------------------------------------------------
# Objetivos:
# a) Compreenso das funes de densidade de probabilidades
# b) Clculo de probabilidades via integrais dessas funes
#-----------------------------------------------------------------------
# Dependncias:
# library(fdth): Frequency Distribution Tables, Histograms and Polygons
#-----------------------------------------------------------------------

#-----------------------------------
#. Exemplos - FDP
#-----------------------------------
#--------------------
#.. Exemplo 0
#--------------------
# Definindo uma fdt: slide
fdp_x <- function(x)
  ifelse(x >= 0 & x <= 2,
         3/8*(4*x - 2*x^2),
          0)

tes <- seq(-1,
           3,
           len=1e3)

# Grfico da funo
plot(fdp_x(x=tes) ~ tes,
     xlab='x',
     ylab='f(x)',
     xlim=c(-1, 3),
     col=2,
     type='l',
     lwd=2)

#... Verificando se fdp_x  uma FDP
# Condio I
fdp_x(seq(-1, 3, by=.1))
fdp_x(seq(-1, 3, by=.1)) >= 0
all(fdp_x(seq(-1, 3, by=.1)) >= 0)

# Condio II
intg_x <- integrate(fdp_x,
                    0,
                    2)
str(intg_x)

intg_x$value

# P(X > 1)
# Grfico
polygon(x=c(seq(1,
                2,
                l=1e2),
            1),
        y=c(fdp_x(seq(1,
                      2,
                      l=1e2)),
            0),
        col=gray(.8))

# Valor
integrate(fdp_x,
          1,
          Inf)$value

# Funo text com as coordenadas de posio do texto
text(x=1.5,
     y=.1,
     expression('P(X>1)=0.5'),
     cex=1.2,
     col='red')

#--------------------
#.. Exemplo 1
#--------------------
# Definindo uma fdt: simples
fdp_y <- function(y)
  ifelse(y >= 0 & y <= 1, 
         2*y, 
         0)

tes <- seq(-1,
           2,
           len=1e3)

# Grfico da funo
plot(fdp_y(y=tes) ~ tes,
     xlab='y',
     ylab='f(y)',
     xlim=c(-1, 2),
     col=2,
     type='l',
     lwd=2)

#... Verificando se fdp_y  uma FDP
# Condio I
fdp_y(seq(-1, 2, by=.1))
fdp_y(seq(-1, 2, by=.1)) >= 0
all(fdp_y(seq(-1, 2, by=.1)) >= 0)

# Condio II
intg_y <- integrate(fdp_y,
                    0,
                    1)
intg_y$value

# Visualizando algumas probabilidades
# Entre 0 e .5
polygon(x=c(seq(0, 
                .5, 
                l=10), 
            .5),
        y=c(fdp_y(seq(0,
                      .5,
                      l=10)),
            0),
        col=gray(.8))

# Entre .5 e 1
polygon(x=c(.5,
            seq(.5, 
                1, 
                l=10), 
            1),
        y=c(0,
            fdp_y(seq(.5,
                      1,
                      l=10)),
            0),
        col=gray(.5))

# Funo text com as coordenadas de posio do texto
text(c(-0.2, 1.3),
     c(.2, .5),
     c(expression(P('0 < Y < .5'), 
                  P('.5 < Y < 1'))),
     cex=1,
     col='red')

# Calculando densidades de probabilidade
# Entre 0 e .5
text(-0.2,
     .1,
     round(integrate(fdp_y,
                     0,
                     .5)$value, 2),
     col='red')

# Entre .5 e 1
text(1.3,
     .4,
     round(integrate(fdp_y,
                     .5,
                     1)$value, 2),
     col='red')

#--------------------
#.. Exemplo 2
#--------------------
# Definindo outra fdt: menos simples
fdp_x <- function(x)
  ifelse(x < 0, 
         0, 
         2*exp(-2*x))

tes <- seq(-1,
           4,
           len=1e3)

# Grfico da funo
plot(fdp_x(x=tes) ~ tes,
     xlab='y',
     ylab='f(y)',
     xlim=c(-1, 4),
     col=2,
     type='l',
     lwd=2)

#... Verificando se fdp_x  uma FDP
# Condio I
all(fdp_x(seq(-1, 100, by=.1)) >= 0)

# Condio II
integrate(fdp_x,
          0,
          Inf)$value

# Visualizando algumas probabilidades
polygon(x=c(1, 
            seq(1, 
                5, 
                l=20)),
        y=c(0, 
            fdp_x(seq(1,
                      5,
                      l=20))),
        col=gray(.5))

polygon(x=c(0.2, 
            seq(0.2, 
                0.8, 
                l=20), 
            0.8),
        y=c(0, 
            fdp_x(seq(0.2,
                      0.8,
                      l=20)),
            0),
        col=gray(.8))

# Funo text com as coordenadas de posio do texto
text(c(1.6, .9),
     c(.3, .8),
     c(expression(Pa, 
                  Pb)),
     col='red')

# Obtendo as probabilidades pedidas
# Questo B
text(1.6, 
     .2, 
     round(integrate(fdp_x,
                     1, 
                     Inf)$value, 2),
     col='red')

# Questo C
text(.9, 
     .7, 
     round(integrate(fdp_x,
                     .2, 
                     .8)$value, 2),
     col='red')


#--------------------------------------------------------------------------
#. PRINCIPAIS DISTRIBUIES
#--------------------------------------------------------------------------
# - Mostrar a igualdade entre a equao apresentada na parte terica com a
#  implementada pelo R
# - Apresentar as funes do R para cada distribuio
#    - Densidade de probabilidade                         (inicia com d)
#    - Probabilidade acumulada                            (inicia com p)
#    - Quantil correspondente a uma dada probabilidade    (inicia com q)
#    - Amostras aleatrias da distribuio                (inicia com r)
#--------------------------------------------------------------------------

#-----------------------------------
#.. Distribuio Normal
#-----------------------------------
# Funo de densidade de probabilidades
# m: mdia
# s: desvio padro

fdp_norm <- function(x,
                     m,
                     s)
  1/(s*sqrt(2*pi))*exp(-1/2*((x-m)/s)^2)

tes <- seq(0, 
           20, 
           len=1e3)

plot(fdp_norm(x=tes,
              m=10,
              s=2) ~ tes,
     type='l',
     lwd=2)

curve(dnorm(x,    # o x aqui  necessrio (mesmo que o objeto no exista)
            m=10,
            s=2),
      col='red',
      lwd=2,
      add=TRUE,
      n=1e3)

# Integrando fdp_norm
integrate(fdp_norm,
          -Inf,
          Inf,
          m=10,
          s=2)$value  # Considerando m=10, s=2

integrate(fdp_norm,
          -Inf,
          10,
          m=10,
          s=2)$value

integrate(fdp_norm,
          10,
          Inf,
          m=10,
          s=2)$value

# Para o R fdp_norm e dnorm so uma mesma funo
#  posssivel integrar (tambm) a funo dnorm
integrate(dnorm,
          -Inf,
          Inf,
          m=10,
          s=2)$value  # Considerando m=10, s=2

integrate(dnorm,
          -Inf,
          10,
          m=10,
          s=2)$value

integrate(dnorm,
          10,
          Inf,
          m=10,
          s=2)$value

# Comparando a integral com uma funo de alto nvel: pnorm
round(integrate(fdp_norm,
                -Inf,
                10,
                m=10,
                s=2)$value, 
      5) ==
round(pnorm(10,
            m=10,
            s=2), 
      5)

# Exemplo - Seja X uma V.A com distribuio N(100,10), calcule
#           as probabilidades:
# a) P[X < 95]
pnorm(95,
     100,
     10)

# b) P[90 < X < 110]
pnorm(110,
      100,
      10) - pnorm(90,
                  100,
                  10)

# c) P[X > 95]
1 - pnorm(95,
          100,
          10)

## TRUE:  Calcula as probabilidades da forma P(X <= x).  
## FALSE: Calcula as probabilidades da forma P(X > x).
pnorm(95,
      100,
      10,
      lower=F)

# Calcular o quantil referente a probabilidade resultante da pnorm
(x = round(pnorm(10,
                 m=10,
                 s=2),
           5))

qnorm(x,
      m=10,
      s=2)

# Calcular o quantil que delimita 40% dos menores valores
# O valor de n tal que P(X <= n) = 0.4
qnorm(40/100,
      m=10,
      s=2)

# Utilizando rnorm para gerar uma amostra de 1e4 ~ N(10, 2)
(am_n <- rnorm(1e4,
               m=10,
               s=2))

library(fdth)
plot(fdt(am_n,
         k=50),
     type='d')

curve(dnorm(x,
            m=10,
            sd=2),
      add=TRUE,
      col=2,
      lwd=2)

# Vrias sobrepostas
plot(NA,
     xlab='x',
     ylab='f(x)',
     xlim=c(-20, 40),
     ylim=c(0, .4),
     type='n')

# Nmero de curvas
nc <- seq(1, 
          10, 
          by=.5)

# Define as cores
cores <- rainbow(length(nc))              

for (i in nc)
  curve(dnorm(x,
              m=10,
              sd=i),
        col=cores[i],
        add=TRUE,
        n=1e3)

#-----------------------------------
#.. Distribuio t
#-----------------------------------
# Funo de densidade de probabilidades
# df: graus de liberdade (n-1)

fdp_t <- function(x,
                  df)
  (gamma((df+1)/2))/(sqrt(df*pi)*gamma(df/2))*((1+x^2/df))^-((df+1)/2)

tes <- seq(-10,
           10,
           len=1e3)

plot(fdp_t(x=tes,
           df=10) ~ tes,
     type='l',
     lwd=2)

curve(dt(x,       #... o x aqui  necessrio (mesmo que o objeto 'x' no exista)
         df=10),
      col='red',
      lwd=2,
      add=TRUE,
      n=1e3)

# Integrando
integrate(fdp_t,
          -Inf,
          Inf,
          df=10)$value  # Considerando df=10

integrate(fdp_t,
          -Inf,
          0,
          df=10)$value

integrate(fdp_t,
          0,
          Inf,
          df=10)$value

# Para o R fdp_t e dt so uma mesma funo
#  posssivel integrar (tambm) a funo dt
integrate(dt,
          -Inf,
          Inf,
          df=10)$value  # Considerando df=10

integrate(dt,
          -Inf,
          0,
          df=10)$value

integrate(dt,
          0,
          Inf,
          df=10)$value

# Comparando a integral com uma funo de alto nvel: pt
round(integrate(fdp_t,
                -Inf,
                0,
                df=10)$value, 
      5) ==
round(pt(0,
         df=10), 
      5)

# Calcular o quantil referente a probabilidade resultante da pt
(x = round(pt(0,
              df=10),
           5))

qt(x,
   df=10)

# Calcular o quantil que delimita 15% dos menores valores
# O valor de t tal que P(X <= t) = 15/100
qt(15/100,
   df=10)

# Utilizando rt para gerar uma amostra de 1e4 ~ t(10)
(am_t <- rt(1e4,
            df=10))
                               
library(fdth)
plot(fdt(am_t,
         k=50),
     type='d')

curve(dt(x,
         df=10),
      add=TRUE,
      col=2,
      lwd=2)

# Vrias sobrepostas
plot(NA,
     xlab='t',
     ylab='f(t)',
     xlim=c(-10, 10),
     ylim=c(0, .4),
     type='n')

# Nmero de curvas
nc <- seq(1, 
          20, 
          by=1)

# Define as cores
cores <- rainbow(length(nc))              

for (i in nc)
  curve(dt(x,
           df=i),
        col=cores[i],
        add=TRUE,
        n=1e3)

# Sobrepondo uma normal padro
curve(dnorm(x),
      lwd=3,
      col=gray(.2),
      add=TRUE,
      n=1e3)

#-----------------------------------
#.. Distribuio Qui-quadrado
#-----------------------------------
# Funo de densidade de probabilidades
# df: graus de liberdade (n-1)

fdp_chisq <- function(x,
                      df)
  (1/(2^(df/2)*gamma(df/2)))*x^(df/2-1)*exp(-x/2)

tes <- seq(0,
           50,
           len=1e3)

plot(fdp_chisq(x=tes,
               df=10) ~ tes,
     type='l',
     lwd=2)

curve(dchisq(x,
             df=10),
      col='red',
      lwd=2,
      add=TRUE,
      n=1e3)

# Integrando
integrate(fdp_chisq,
          0,
          Inf,
          df=10)$value  # Considerando df=10

(p1 <- integrate(fdp_chisq,
                 0,
                 20,
                 df=10)$value)

(p2 <- integrate(fdp_chisq,
                 20,
                 Inf,
                 df=10)$value)

round(p1 + p2, 5) == 1

# Para o R fdp_chisq e dchisq so uma mesma funo!
# Assim,  posssivel integrar (tambm) a funo dchisq
integrate(dchisq,
          0,
          Inf,
          df=10)$value  # Considerando df=10

integrate(dchisq,
          0,
          20,
          df=10)$value

integrate(dchisq,
          20,
          Inf,
          df=10)$value

# Comparando a integral com uma funo de alto nvel: pchisq
round(integrate(fdp_chisq,
                0,
                20,
                df=10)$value,
      5) ==
round(pchisq(20,
             df=10),
      5)

# Calcular o quantil referente a probabilidade resultante da pchisq
# pchisq e qchisq so funes inversas
(x <-  round(pchisq(20,
                    df=10),
             5))

qchisq(x,
       df=10)

# Calcular o quantil que delimita 50% dos menores valores
# O valor de q tal que P(X <= q) = 50/100 (df=10)
qchisq(50/100,
       df=10)

# Utilizando rchisq para gerar uma amostra de 1e4 ~ Qui-quadrado(10)
(a_qq <- rchisq(1e4,
                df=10))

library(fdth)
plot(fdt(a_qq,
         k=50),
     type='d')

curve(dchisq(x,
         df=10),
      add=TRUE,
      col=2,
      lwd=2)

# Vrias sobrepostas
plot(NA,
     xlab='X2',
     ylab='f(X2)',
     xlim=c(0, 80),
     ylim=c(0, .5),
     type='n')

# Nmero de curvas
nc <- 40

# Define as cores
cores <- rainbow(nc)

for (i in 1:nc)
  curve(dchisq(x,
               df=i),
        col=cores[i],
        add=TRUE,
        n=1e3)

#-----------------------------------
#.. Distribuio F
#-----------------------------------
# Funo de densidade de probabilidades
# df1: graus de liberdade do numerador (n-1)
# df2: graus de liberdade do denominador (n-1)

fdp_f <- function(x,
                  df1,
                  df2)
  ((gamma((df1+df2)/2))/(gamma(df1/2)*gamma(df2/2)))*((df1/df2)^(df1/2))*((x^((df1-2)/2))/((1+(df1/df2)*x)^((df1+df2)/2)))

tes <- seq(0, 
           10, 
           len=1e3)

plot(fdp_f(x=tes,
           df1=5,
           df2=15) ~ tes,
     type='l',
     lwd=2)

curve(df(x,
         df1=5,
         df2=15),
      col='red',
      lwd=2,
      add=TRUE,
      n=1e3)

# Integrando
integrate(fdp_f,
          0,
          Inf,
          df1=5,
          df2=15)$value  # Considerando df1=5, df2=15

(p1 <- integrate(fdp_f,
                 0,
                 3,
                 df1=5,
                 df2=15)$value)

(p2 <- integrate(fdp_f,
                 3,
                 Inf,
                 df1=5,
                 df2=15)$value)

round(p1 + p2, 5) == 1

# Para o R fdp_f e df so uma mesma funo
#  posssivel integrar (tambm) a funo df
integrate(df,
          0,
          Inf,
          df1=5,
          df2=15)$value  # Considerando df1=5, df2=15

integrate(df,
          0,
          3,
          df1=5,
          df2=15)$value

integrate(df,
          3,
          Inf,
          df1=5,
          df2=15)$value

# Comparando a integral com uma funo de alto nvel: pf
round(integrate(fdp_f,
                0,
                3,
                df1=5,
                df2=15)$value, 
      5) ==
round(pf(3,
         df1=5,
         df2=15), 
      5)

# Calcular o quantil referente a probabilidade resultante da pf
(x = round(pf(3,
              df1=5,
              df2=15),
           5))

qf(x,
   df1=5,
   df2=15)

# Calcular o quantil que delimita 60% dos menores valores
# O valor de f tal que P(X <= f) = 0.6
qf(60/100,
   df1=5,
   df2=15)

# Utilizando rf para gerar 1e4 ~ F(df1=5, df2=15)
(am_f <- rf(1e4,
            df1=5,
            df2=15))

library(fdth)
plot(fdt(am_f,
         k=50),
     type='d')

curve(df(x,
         df1=5,
         df2=15),
      add=TRUE,
      col=2,
      lwd=2,
      n=1e3)

# Vrias sobrepostas
plot(NA,
     xlab='F',
     ylab='f(F)',
     xlim=c(0, 10),
     ylim=c(0, 1.2),
     type='n')

# Nmero de curvas
nc <- 30

# Define as cores
cores <- rainbow(nc)              

for (i in 1:nc)
  curve(df(x,
           df1=i,    # Variando apenas df1
           df2=15),
        col=cores[i],
        add=TRUE,
        n=1e3)

#plot(NA,
#     xlab='F',
#     ylab='f(F)',
#     xlim=c(0, 10),
#     ylim=c(0, 1.2),
#     type='n')
#
#for (i in 1:10)
#  for (j in 1:nc)
#    curve(df(x,
#             df1=i,    # Variando df1 e df2
#             df2=j),
#          col=cores[j],
#          add=TRUE,
#          n=1e3)
